/*
 * Copyright (c) 2005-2013, BearWare.dk
 * 
 * Contact Information:
 *
 * Bjoern D. Rasmussen
 * Bedelundvej 79
 * DK-9830 Taars
 * Denmark
 * Email: contact@bearware.dk
 * Phone: +45 20 20 54 59
 * Web: http://www.bearware.dk
 *
 * This source code is part of the TeamTalk 4 SDK owned by
 * BearWare.dk. All copyright statements may not be removed 
 * or altered from any source distribution. If you use this
 * software in a product, an acknowledgment in the product 
 * documentation is required.
 *
 */

#include "userinfodlg.h"
#include "mainwindow.h"

extern TTInstance* ttInst;

UserInfoDlg::UserInfoDlg(int userid, QWidget * parent/* = 0*/)
    : QDialog(parent, Qt::Dialog | Qt::WindowTitleHint)
, m_userid(userid)
{
    ui.setupUi(this);
    setWindowIcon(QIcon(APPICON));

    startTimer(250);
    updateUser();
}

void UserInfoDlg::timerEvent(QTimerEvent *)
{
    updateUser();
}

void UserInfoDlg::updateUser()
{
    User user;
    UserStatistics stats;
    if(!TT_GetUser(ttInst, m_userid, &user))
        return;

    if(!TT_GetUserStatistics(ttInst, m_userid, &stats))
        return;

    ui.userid->setText(QString::number(user.nUserID));
    if(ui.nickname->text() != _Q(user.szNickname))
        ui.nickname->setText(_Q(user.szNickname));
    switch(user.nStatusMode & STATUSMODE_MODE)
    {
    case STATUSMODE_AVAILABLE :
        ui.statusmode->setText(tr("Available")); break;
    case STATUSMODE_AWAY :
        ui.statusmode->setText(tr("Away")); break;
    case STATUSMODE_QUESTION :
        ui.statusmode->setText(tr("Question")); break;
    default :
        ui.statusmode->setText(tr("Unknown")); break;
    }

    if(ui.statusmsg->text() != _Q(user.szStatusMsg))
        ui.statusmsg->setText(_Q(user.szStatusMsg));
    if(ui.username->text() != _Q(user.szUsername))
        ui.username->setText(_Q(user.szUsername));

    switch(user.uUserType)
    {
    case USERTYPE_ADMIN :
        ui.usertype->setText(tr("Administrator"));break;
    case USERTYPE_DEFAULT :
        ui.usertype->setText(tr("Default")); break;
    default:
        ui.usertype->setText(tr("Unknown")); break;
    }

    if(TT_GetMyUserType(ttInst) & USERTYPE_ADMIN)
    {
        if(ui.ipaddr->text() != _Q(user.szIPAddress))
            ui.ipaddr->setText(_Q(user.szIPAddress));
    }
    else
        ui.ipaddr->setText(QString());
    ui.p2p->setText(user.uUserState & USERSTATE_P2P_CONNECTED? tr("Yes") : tr("No"));
    if(ui.version->text() != _Q(user.szVersion))
        ui.version->setText(_Q(user.szVersion));

    ui.audpacketloss->setText(QString("%1/%2").arg(stats.nAudioPacketsLost).arg(stats.nAudioPacketsRecv+stats.nAudioPacketsLost));
    ui.vidpacketloss->setText(QString("%1/%2").arg(stats.nVideoFramesLost).arg(stats.nVideoFramesRecv+stats.nVideoFramesLost));
}
